/**
 * Middleware IP Whitelist
 *
 * Middleware ini memeriksa apakah IP pengguna terdaftar dalam whitelist
 * untuk mengakses panel admin. Jika tidak, pengguna akan dialihkan ke halaman 404.
 */

const fs = require("fs");
const path = require("path");

// Path ke file JSON whitelist
const dataDir = path.join(__dirname, "data");
const whitelistPath = path.join(dataDir, "ip-whitelist.json");
const logPath = path.join(dataDir, "ip-whitelist.log");

// Function to write logs to a file
function writeLog(message) {
  try {
    // Create date with timezone
    const now = new Date().toISOString();
    const logMessage = `[${now}] ${message}\n`;
    
    // Make sure data directory exists
    if (!fs.existsSync(dataDir)) {
      fs.mkdirSync(dataDir, { recursive: true });
    }
    
    // Append to log file
    fs.appendFileSync(logPath, logMessage);
  } catch (error) {
    // Can't do much if logging itself fails
    console.error("Error writing to log file:", error);
  }
}

// Fungsi untuk mendapatkan daftar IP yang diizinkan
function getIpWhitelist() {
  try {
    writeLog("Attempting to read whitelist file");
    
    // Jika file whitelist ada, baca dari file
    if (fs.existsSync(whitelistPath)) {
      writeLog(`Whitelist file exists at ${whitelistPath}`);
      const data = fs.readFileSync(whitelistPath, "utf8");
      writeLog(`Whitelist content: ${data}`);
      return JSON.parse(data);
    }

    // Buat direktori data jika belum ada
    if (!fs.existsSync(dataDir)) {
      writeLog(`Creating data directory at ${dataDir}`);
      fs.mkdirSync(dataDir, { recursive: true });
    }

    // Fallback ke daftar default jika file tidak ada
    const defaultList = [
      "127.0.0.1", // Localhost
      "::1", // Localhost IPv6
      "192.168.0.*", // Semua IP dalam jaringan 192.168.0.x
      "192.168.1.*", // Semua IP dalam jaringan 192.168.1.x
    ];

    writeLog(`Whitelist file not found, creating default: ${JSON.stringify(defaultList)}`);
    
    // Tulis file default
    fs.writeFileSync(
      whitelistPath,
      JSON.stringify(defaultList, null, 2),
      "utf8"
    );

    return defaultList;
  } catch (error) {
    writeLog(`ERROR reading whitelist: ${error.message}`);
    // Fallback ke daftar default jika terjadi error
    return ["127.0.0.1", "::1"];
  }
}

// Middleware untuk memeriksa IP whitelist
const checkIpWhitelist = (req, res, next) => {
  writeLog("-------- New IP check request --------");
  writeLog(`Request URL: ${req.originalUrl}`);
  
  // Log all headers to diagnose proxy issues
  writeLog(`Request headers: ${JSON.stringify(req.headers)}`);
  
  // Get the client IP - improved for cPanel/proxy environments
  let clientIp;
  
  if (req.headers['x-forwarded-for']) {
    clientIp = req.headers['x-forwarded-for'].split(',')[0].trim();
    writeLog(`IP from x-forwarded-for: ${clientIp} (from full header: ${req.headers['x-forwarded-for']})`);
  } else if (req.headers['x-real-ip']) {
    clientIp = req.headers['x-real-ip'];
    writeLog(`IP from x-real-ip: ${clientIp}`);
  } else if (req.ip) {
    clientIp = req.ip;
    writeLog(`IP from req.ip: ${clientIp}`);
  } else if (req.connection?.remoteAddress) {
    clientIp = req.connection.remoteAddress;
    writeLog(`IP from req.connection.remoteAddress: ${clientIp}`);
  } else if (req.socket?.remoteAddress) {
    clientIp = req.socket.remoteAddress;
    writeLog(`IP from req.socket.remoteAddress: ${clientIp}`);
  } else {
    clientIp = "unknown";
    writeLog("Could not detect IP address from any source");
  }

  // Clean the IP address
  const ipAddress = (clientIp || "").replace(/^::ffff:/, "");
  writeLog(`Final cleaned IP address: ${ipAddress}`);

  // Dapatkan daftar IP yang diizinkan dari file
  const ipWhitelist = getIpWhitelist();
  writeLog(`Loaded whitelist: ${JSON.stringify(ipWhitelist)}`);

  // Fungsi untuk memeriksa IP dengan wildcard pattern
  const ipMatchesPattern = (ip, pattern) => {
    writeLog(`Checking if IP ${ip} matches pattern ${pattern}`);
    
    // IPv6 localhost
    if (
      ip === "::1" &&
      (pattern === "::1" || pattern === "localhost" || pattern === "127.0.0.1")
    ) {
      writeLog(`IPv6 localhost match for ${ip}`);
      return true;
    }

    // Jika pattern memiliki wildcard (*)
    if (pattern.includes("*")) {
      const patternParts = pattern.split(".");
      const ipParts = ip.split(".");

      // Memastikan jumlah bagian IP sama (IPv4)
      if (ipParts.length !== 4 || patternParts.length !== 4) {
        writeLog(`Parts mismatch: IP parts (${ipParts.length}): ${ipParts.join('.')}, Pattern parts (${patternParts.length}): ${patternParts.join('.')}`);
        return false;
      }

      // Periksa setiap bagian dari IP
      let matches = true;
      for (let i = 0; i < 4; i++) {
        if (patternParts[i] !== "*" && patternParts[i] !== ipParts[i]) {
          writeLog(`Part ${i+1} mismatch: IP=${ipParts[i]}, Pattern=${patternParts[i]}`);
          matches = false;
          break;
        }
      }
      
      if (matches) {
        writeLog(`Wildcard pattern match for ${ip} with pattern ${pattern}`);
      }
      return matches;
    }

    // Jika tidak ada wildcard, periksa kecocokan langsung
    const matches = ip === pattern;
    if (matches) {
      writeLog(`Direct match for ${ip} with pattern ${pattern}`);
    }
    return matches;
  };

  // Periksa apakah IP ada dalam whitelist
  let ipAllowed = false;
  for (const pattern of ipWhitelist) {
    if (ipMatchesPattern(ipAddress, pattern)) {
      ipAllowed = true;
      break;
    }
  }

  if (ipAllowed) {
    // IP diizinkan, lanjutkan ke middleware berikutnya
    writeLog(`IP ${ipAddress} ALLOWED. Proceeding to next middleware.`);
    return next();
  }

  // Jika IP tidak dalam whitelist, kirim ke halaman 404
  writeLog(`IP ${ipAddress} DENIED ACCESS. Returning 404 page.`);
  try {
    return res.status(404).render("404", {
      message: "Halaman tidak ditemukan",
    });
  } catch (error) {
    writeLog(`ERROR rendering 404 page: ${error.message}`);
    return res.status(404).send("Halaman tidak ditemukan");
  }
};

module.exports = checkIpWhitelist;