// Profile Routes Module
const express = require('express');
const router = express.Router();
const fs = require('fs');
const path = require('path');
const bcrypt = require('bcryptjs');
const multer = require('multer');

// Setup multer untuk upload file
const storage = multer.diskStorage({
    destination: function (req, file, cb) {
        const uploadDir = path.join(__dirname, 'public', 'uploads');
        if (!fs.existsSync(uploadDir)) {
            fs.mkdirSync(uploadDir, { recursive: true });
        }
        cb(null, uploadDir);
    },
    filename: function (req, file, cb) {
        cb(null, Date.now() + path.extname(file.originalname));
    }
});

const upload = multer({ 
    storage: storage,
    limits: { fileSize: 5 * 1024 * 1024 } // 5MB limit
});

// Path ke file users.json
const usersPath = path.join(__dirname, 'data', 'users.json');

// Route untuk melihat profil
router.get('/profile', (req, res) => {
    try {
        // Baca file users.json
        const usersData = fs.readFileSync(usersPath, 'utf8');
        const users = JSON.parse(usersData);
        
        // Temukan data user berdasarkan username
        const userData = users.find(user => user.username === req.session.user.username);
        
        if (!userData) {
            req.flash('error_msg', 'Data pengguna tidak ditemukan');
            return res.redirect('/admin/dashboard');
        }
        
        res.render('admin/profile', {
            userData,
            userRole: req.session.userRole,
            success_msg: req.flash('success_msg'),
            error_msg: req.flash('error_msg')
        });
    } catch (error) {
        console.error('Error accessing profile:', error);
        req.flash('error_msg', 'Terjadi kesalahan saat mengakses halaman profil');
        res.redirect('/admin/dashboard');
    }
});

// Route untuk update profil
router.post('/profile/update', upload.single('profileImage'), async (req, res) => {
    const { displayName, email, bio } = req.body;
    
    try {
        // Get user data
        const users = JSON.parse(fs.readFileSync(usersPath, 'utf8'));
        const userIndex = users.findIndex(u => u.id === req.session.user.id);
        
        if (userIndex === -1) {
            req.flash('error_msg', 'User tidak ditemukan');
            return res.redirect('/admin/profile');
        }
        
        // Update basic info
        users[userIndex].displayName = displayName;
        users[userIndex].email = email;
        users[userIndex].bio = bio;
        
        // Process profile image if uploaded
        if (req.file) {
            const originalPath = path.join(uploadsPath, req.file.filename);
            const targetFilename = req.file.filename + '.jpg';
            const targetPath = path.join(uploadsPath, targetFilename);
            
            // Proses gambar ke ukuran yang tepat (opsional)
            const success = await processImage(originalPath, targetPath);
            if (success) {
                // Save profile image path
                users[userIndex].profileImage = `/uploads/${targetFilename}`;
            }
        }
        
        // Save updated user data
        fs.writeFileSync(usersPath, JSON.stringify(users, null, 2), 'utf8');
        
        // Update session
        req.session.user = users[userIndex];
        
        req.flash('success_msg', 'Profil berhasil diperbarui');
        return res.redirect('/admin/profile?success=true&action=profile');
    } catch (err) {
        console.error('Error updating profile:', err);
        req.flash('error_msg', 'Terjadi kesalahan saat memperbarui profil');
        return res.redirect('/admin/profile?success=false&error=Terjadi kesalahan saat memperbarui profil');
    }
});

// Route untuk perubahan password
router.post('/profile/change-password', (req, res) => {
    const { currentPassword, newPassword, confirmPassword } = req.body;
    
    try {
        // Basic validation
        if (!currentPassword || !newPassword || !confirmPassword) {
            req.flash('error_msg', 'Semua field wajib diisi');
            return res.redirect('/admin/profile?success=false&error=Semua field wajib diisi');
        }
        
        if (newPassword !== confirmPassword) {
            req.flash('error_msg', 'Password baru dan konfirmasi tidak sama');
            return res.redirect('/admin/profile?success=false&error=Password baru dan konfirmasi tidak sama');
        }
        
        // Read users data
        const users = JSON.parse(fs.readFileSync(usersPath, 'utf8'));
        const userIndex = users.findIndex(u => u.id === req.session.user.id);
        
        if (userIndex === -1) {
            req.flash('error_msg', 'User tidak ditemukan');
            return res.redirect('/admin/profile?success=false&error=User tidak ditemukan');
        }
        
        // Verify current password
        const user = users[userIndex];
        const isMatch = bcrypt.compareSync(currentPassword, user.password);
        
        if (!isMatch) {
            req.flash('error_msg', 'Password saat ini tidak benar');
            return res.redirect('/admin/profile?success=false&error=Password saat ini tidak benar');
        }
        
        // Hash and update new password
        const salt = bcrypt.genSaltSync(10);
        const hashedPassword = bcrypt.hashSync(newPassword, salt);
        
        // Update user data
        users[userIndex].password = hashedPassword;
        
        // Save changes
        fs.writeFileSync(usersPath, JSON.stringify(users, null, 2), 'utf8');
        
        req.flash('success_msg', 'Password berhasil diubah');
        return res.redirect('/admin/profile?success=true&action=password');
    } catch (err) {
        console.error('Error changing password:', err);
        req.flash('error_msg', 'Terjadi kesalahan saat mengubah password');
        return res.redirect('/admin/profile?success=false&error=Terjadi kesalahan saat mengubah password');
    }
});

module.exports = router; 